# SYSTYLE

A design system builder for declarative view libs, in the spirit of styled-system but with its own css management.

Add it to your project with `npm install systyle`

## Presets

To make your life easier, systyle comes with a couple presets to have a working styled system for various view libs. They also come with a set of already implemented system rules so you can start styling right away. You can see which libs are available right now at [src/presets](src/presets) and which css rules are implemented at [src/helpers/rules.js](src/helpers/rules.js).

Example for React:

```JS
import ReactSystem from 'systyle/lib/presets/react'

// start styling right away
const Box = ReactSystem.extend({ display: 'flex' })
```

## system(rules: object)

You can also create a system from scratch by calling createStyleSystem and passing it a renderer function adapted to your view lib.
It allows you to define de basic rules of your system by returning the basic component on which all the others will be based.

Every key you define in the rule object can be used as a prop on a system component. When used, it will generate the css related to the rule defined.

If you pass anything that's not a string nor a function, the rule key will be kebab cased and used as css prop.

If you define it as a string, this string will be converted to kebab case and used as css prop inside your stylesheet.

If you use a function, it will be executed when the component renders and receive the current value for the rule prop along with all the other props received by the component. If the returned value is an object, it will be evalutated with the system rules as well, and then deeply merged with the initally passed props before actually building the css with them. This means that if you want your functional rule to actually generate styling, they will have to return an object which keys match some other rules defined in the system. Every generated key that does not have a matching rule won't be considered a styling prop and will consequently be passed as as normal prop to the rendered component.

In the special case where a functional rule returns an object referencing this same rule, the system won't try to apply rules to it (and dive into an infinite loop) but rather consider that the result is already a styling prop. This allows you to build functional rules without necessarily having to define another simpler rule that would deal with actually generating the styling.

```JS
import { createStyleSystem } from 'systyle'
import { renderer } from 'systyle/lib/presets/react'

const system = createStyleSystem(renderer)

const System = system({
  // basic
  display: true,

  // aliased
  pt: 'padding-top',
  pb: 'padding-bottom',

  // functional returning other rule props
  py: value => ({ pt: value, pb: value }),

  // base + functional using props
  backgroundColor: true,
  bg: (value, props) => ({ backgroundColor: props.theme.colors[value] || value}),

  // self referencing funtional rule
  borderWidth: value => ({ borderWidth: value + 'px' })
})
```

## System

The basic system component can be passed any prop defined in the rules and will trigger the generation of the related css.

Note that props passed to a system component will be filtered before being passed to the system renderer. That way, you can be sure that none of the props specific to the system will leak and for example be applied to your DOM elements.

```JS
// using the rules from above:

// a hidden div
<System display="none" />

// a div with a padding-top of 10px
<System pt={10} />

// a div with a padding-top and padding-bottom of 10px
<System pt={10} pb={10} />

// a div with a padding-top and padding-bottom of 10px
<System py={10} />
```

## as

The `as` prop allows you to change which component will be rendered at the root of your system component. By default, it will be `div`.

```JS
// renders a span
<System as="span" />

// you can also compose and even use other system components
<System as={MyComponent} />
```

## System.extend(props: object)

You can extend components using the `.extend()` function. The props you pass it will be used as default props for this newly created component. You'll be able to overwrite them by passing props directly when rendering the component.

It allows you to quickly define the main elements of your system, as well as variations of a component. The returned component will also be a system component so you'll be able to use `.extend()` on it.

If you base a component out of another but you don't want one of its styling props to be applied, just override it by passing it `null` in the new props object.

```JS
// using a system that has all the font rules defined

const Text = System.extend({ as:'span', fontSize: 12, fontFamily: 'Futura' })
const Title = Text.extend({ as:'h1', fontSize: 22, fontWeight: 'bold', textDecoration: 'underline' })
const Subtitle = Title.extend({ as: 'h2', fontSize: 18, textDecoration: null })

// they will all have the same font family
// Title and Subtitle will be bold
// Title will be underlined but Subtitle won't
// they all have different font sizes and dom element

// but you can still change it if you want
<Title fontSize={32} />

// another example with variations
const Button = System.extend({
  as: 'button',
  bg: 'transparent',
  border: '2px solid black',
  color: 'black'
})

Button.danger = Button.extend({
  borderColor: 'red',
  color: 'red'
})

// will give you a red button
<Button.danger>DELETE</Button.danger>
```

If you define one of the props as a function, it will be executed before rendering the component, so that you can compute props dynamically based on the passed props.

```JS
const Box = System.extend({
  display: props => props.hidden ? 'none' : 'block'
})

// the box will have display: block
<Box />

// the box will have display: none
<Box hidden />

// you can also pass functions to non-system related props

const SaveButton = Button.extend({
  disabled: props => props.loading || props.disabled,
  children: props => props.loading ? 'Loading...' : 'Save'
})

// will render a button with 'Save' written inside
<SaveButton />

// will render a disabled button with 'Save' written inside
<SaveButton disabled />

// will render a disabled button with 'Loading...' written inside
<SaveButton loading />
```

## System.system(rules: object)

Every system can also create a new system with specific extra rules. This makes you able to generate localized rules that will only apply to certain components.

Same as `.extend()`, you can call this function from any component generated through this lib. The new component will also inherit the default props of its parent, as well as its wrappers (see next section).

```JS
const Base = system({
  display: true,
  flex: true,
  flexDirection: true,
  justifyContent: true,
  alignItems: true,
  alignSelf: true
})

const Sticker = Base.system({
  position: true,
  top: true,
  right: true,
  bottom: true,
  left: true,
  bounds: ([x, y, width, height]) => ({
    position: 'fixed',
    left: x,
    top: y,
    width: width,
    height: height
  })
})

const Text = Base.system({
  fontFamily: true,
  fontSize: true,
  fontWeight: true
})

// you can now apply the rules defined in the first system and the new ones

<Sticker display="flex" bounds={[0, 0, 50, 100]} />

<Text display="inline-flex" fontSize={15}>
```

## System.with(component: Component)

In some occasions, you may want to enhance a basic component with systyle methods. One way of doing it would be to use the composition power of the `as` prop of a system component, however this prop is easily overwritten. That's where the `.with()` hoc comes to the rescue by allowing you to wrap any component with system functions and making sure that this component will always be rendered as the root of this component instead of `as`, even after you use `.extend()` or any other systyle method on it.

```JS
const Form = ({ email, onChange, className }) => (
  <form className={className}>
    <input name="email" value={email} onChange={onChange} />
  </form>
)

// compose with as
const SystemAsForm = System.extend({ as: Form })

// but the Form component is completely lost if you do something like that
<SystemAsForm as="div" display="flex" flexDirection="column" />

// using .with() prevents that problem
const SystemWithForm = System.with(Form)

// as will be ignored here because Form doesn't do anything about this prop
// but the other props will still generate styling and create class names
<SystemWithForm as="div" display="flex" flexDirection="column" />
```

## System.compose(...wrappers: [function])

The `.compose()` method allows you to wrap your system component with higher order components. By calling directly `hoc(System)`, you won't be able to use systyle methods like `.extend()` on the resulting component. `.compose()` fixes that issue by generating a system component, except that it'll be rendered wrapped with the HOCs. One other major difference is also that any component built out of a composed system component will also be wrapped with the same HOC. If you use `.compose()` on an already composed component, the new wrapper will be applied on top of the existing one.

```JS
// create a system component that will rerender only when its props change
const Memoized = System.compose(React.memo)

// this component will also be wrapped with React.memo
const MemoizedButton = Memoized.extend({ as: 'button' })

// create a hoc that consumes a theme context
const withTheme = Component => props => { /* pass theme as prop to the wrapped component */ }

// the component will we wrapped by React.memo + withTheme
const ThemedMemoizedButton = MemoizedButton.compose(withTheme).extend({
  bg: props => props.theme.background
})

// You can also write it in one command
const ThemedMemoizedButton = System.compose(React.Memo, withTheme).extend({
  bg: props => props.theme.background
})
```

## System.animate(timing: string, keyframes: object)

You can generate animated components by using the `.animate()` function. It expects 2 params, the first being the timing details of the animation (function, delay, duration, etc) and the 2nd being the actual keyframes describing the transformation.

As the animation is generated through the system, you can describe your keyframes by using system rules that will ultimately be converted to the corresponding styling. Using a prop that doesn't have a matching rule in the system won't generate any CSS.

Because the animate function generates a new `animation` prop as part of the default props of the component, you'll have to make sure that your system implements this rule, otherwise no css will be generated and nothing will move. Note also that as you'll be generating keyframes, you can only use styling rules. Any rule that returns something like pseudo classes or non-styling props won't have any influence on the outcome.

You can eventually get the name of the generated keyframe by accessing the `animationName` attribute of your new component in case you would like to use it to define animations in other, normally unrelated places.

If you use `.animate()` on an already animated component, animations will stack.

```JS
const System = system({
  animation: true,
  transform: true,
  slideY: value => ({ transform: `translateY(${value}%)` })
})

const SlideUp = System.animate('1s ease-in', {
  '0%': { slideY: 100 },
  '100%': { slideY: 0 }
})

// this component will slide up when mounted
<SlideUp />

// and this is equivalent to writing
<System animation={`${SlideUp.animationName} 1s ease-in`} />
```

By using specific css animation props, you can easily design a subsystem that controls your animations.

```JS
const System = system({
  transform: true,
  rotate: value => ({ transform: `rotate(${value}deg)`})
})

const Animated = System.system({
  animation: true,
  animationPlayState: true
  pause: value => ({ animationPlayState: value ? 'paused' : 'running' })
})

const Rotating = Animated.animate('1s infinite', {
  from: { rotate: 0 },
  to: { rotate: 360 }
})

// rotates forever
<Rotating />

// stays still
<Rotating pause />
```

## omit

The `omit` prop is there for you to list any prop that you don't want to see passed to the rendered component. The props listed there will still participate in all system computations, they will just be filtered out at render time.

```JS
const AsyncButton = System.extend({
  as: 'button',
  disabled: p => p.loading || p.disabled, // loading is not part of the system so it'll normally be passed to the root component
  omit: ['loading'] // but you can filter it out before it happens
})
```

## Pseudo selectors

In order to deal with pseudo selectors, you can use the `$pseudo` props.
You're supposed to pass it an object which keys are a pseudo class. As values, you should give an object containing rule values, similar to the ones you would normally pass to the component to style it.

```JS
const WithHover = System.extend({
  $pseudo: { ':hover': { bg: 'red' } }
})
```

TIP: you can define functional rules that will make writing this easier

```JS
const System = system({
  $hover: value => ({ $pseudo: { ':hover': value }})
})

// will have a red bg on hover
<System $hover={{ bg: 'red' }}>
```

## Media queries

You can also specify media queries using the `$wrap` prop. You'll have to pass it an object of which keys will be media queries along with the corresponding style object.

```JS

const WithMediaQuery = System.extend({
  $wrap: { '@media screen and (min-width: 300px)': { bg: 'red' } }
})

// you can also nest them
const WithHoverMediaQuery = System.extend({
  $wrap: {
    '@media screen and (min-width: 300px)': {
      $pseudo: {
        ':hover': { bg: 'red' }
      }
    }
  }
})
```

## Children rules

In some cases, it is still useful to have the ability to define rules for many children in one place. In this situation, you can use the `$children` prop when defining or rendering a component. The object you will give it will be resolved through the rules of your system and will finally generate a style that will apply to the specified children of your component.

The keys of this object should be a valid CSS selector. The raw string that you wrote will be used to define your rule, prefixed by a unique class name that will identidfy this rule and be passed to the parent component. If there are many selectors inside (eg: `div, span, li`) each of them will be scoped as well. On top of that, every system component will be given a unique class name so you can identify them in the child rules by directly using them as key.

As values, you can pass any prop matching rules you defined in your system. Note however that meta rules will be completely ignored.

```JS
const System = system({ bg: 'background' })

const Button = System.extend({ as: 'button' })

const List = System.extend({
  $children: {
    li: { bg: 'red' },
    [Button]: { bg: 'white' }
  }
})

// all the li inside will have a red bg
// the button will have a white background
const myLi = (
  <List>
    <li>hi</li>
    <Button>bye</Button>
  </List>
)
```

## Inline styles

Inline styles can be very useful when for example you need to drag an element on screen and update its position on every `mousemove`. If you don't use inline styles in this kind of case, the lib will generate a crazy amount of CSS rules and probably crash everything.

You have two ways to access the inline styles: from the system level or from the component level.

At system level, you can define functional rules that return an object with a `style` key inside. As every object generated by a functional rule is merged in the final list of props that will be passed to your rendered component, it will get the style you created from your rule.

```JS
const Draggable = system({
  position: true,
  x: value => ({ style: { left: value + 'px' }}),
  y: value => ({ style: { top: value + 'px' }})
}).extend({ position: 'absolute' })

// <div class="qwDUwId9" style="left: 325px; top: 530px"></div>
<Draggable x={325} y={530} />
```

At component level, you have to take advantage of functional props and pass a function to the `style` key of your props object. There is a however a slight disadvantage to this technique as the system won't be able to intercept which props are used by the function so if you do not manually specify those props in the `omit` prop, they will always end up being passed down to the rendered component.

```JS
const Draggable = System.extend({
  style: props => ({ left: props.x + 'px', top: props.y + 'px' })
  omit: ['x', 'y']
})

// <div class="qwDUwId9" x="325" y="325" style="left: 325px; top: 530px"></div>
<Draggable x={325} y={530} />
```

## About CSS management

To avoid storing too many rules that often look like each other, this lib chooses instead to dedicate one specific class name for one specific value of a css prop, a bit like Atomic CSS. Once generated, those class names will be cached for faster later retrieval and guarantee that there won't be any duplicate rule. Finally, the system components will receive only the classes that match the style configuration dictated by their props at a given time.
