"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.zero = zero;
exports.percent = percent;
exports.pixel = pixel;
exports.space = space;
exports.color = color;
exports.font = font;
exports.responsive = responsive;

var _typeof2 = _interopRequireDefault(require("@babel/runtime/helpers/typeof"));

var _utils = require("../../utils");

var BREAKPOINTS = [0, 320, 768, 1224];

function mediaQuery(key, breakpoints) {
  return key in breakpoints ? "@media screen and (min-width: ".concat(breakpoints[key], "px)") : key; // prettier-ignore
}

function theme(props, path, defaults) {
  return (0, _utils.get)(props, ['theme'].concat(path)) || defaults;
} // returns 0 as a string if the value is 0


function zero(value, props, next) {
  return next(value === 0 ? '0' : value);
} // return the value as a percentage if it's between 0 and 1


function percent(value, props, next) {
  return next(0 < value && value <= 1 ? (value * 100).toFixed(2) + '%' : value);
} // append px to the value if it's a number


function pixel(value, props, next) {
  return next(typeof value === 'number' ? value + 'px' : value);
} // it it's a number, multiply the value by the defined spacing


function space(value, props, next) {
  var size = theme(props, 'spacing', 8);
  return next(typeof value === 'number' ? value * size : value);
} // check if the given color is defined in the theme


function color(value, props, next) {
  return next(theme(props, ['colors', value], value));
} // check if the given font family is defined in the theme


function font(value, props, next) {
  return next(theme(props, ['fonts', value], value));
} // generate media queries based on the breakpoints defined in the theme
//
// breakpoints should be defined as an object (or array)
// if the breakpoint value is a number it will be converted to a special media query
// if not, it will be used as is
//
// when using the transformer, the given value should be on object matching defined breakpoints
//
// example:
// const breakpoints = { mobile: 300, tablet: 800, desktop: 1200 }
//
// const props = { theme: { breakpoints } }
// const next = v => v
//
// const value = {
//   mobile: { color: 'red' },
//   desktop: { color: 'green' }
// }
//
// const mediaQueries = responsive(value, props, next)


function responsive(value, props, next) {
  if (!value || (0, _typeof2.default)(value) !== 'object') {
    return next(value);
  }

  var result = {};
  var breakpoints = theme(props, 'breakpoints', BREAKPOINTS);
  (0, _utils.forEach)(value, function (mqValue, mq) {
    var mqResult = next(mqValue);

    if (breakpoints[mq] === 0) {
      Object.assign(result, mqResult);
    } else {
      result.$wrap || (result.$wrap = {});
      result.$wrap[mediaQuery(mq, breakpoints)] = mqResult;
    }
  });
  return result;
}
//# sourceMappingURL=transform.js.map